<?php

namespace Drupal\kashing\Entity;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;

class KashingAPI
{

    private $mode;

    private $merchant_id;

    private $secret_key;

    private $currency;

    private $api_url;

    private $has_errors;

    private $errors;

    private $config;

    private $redirect_url;

    private $response_message;


    function __construct() {

        $this->has_errors = false;
        $this->errors = array();
        $this->config = \Drupal::config('kashing.settings');

        if(!isset($this->config)) {
            $this->add_error(array(
                'field' => 'settings',
                'type' => 'missing_data',
                'msg' => t('The kashing settings data is missing.')
            ));
        } else {
            $this->init_configuration();
        }

    }


    private function init_configuration() {

        $this->mode = $this->config->get('mode');

        //determine api mode (test default)

        if ($this->mode == 'live') {
            $option_mode = 'live';
            $this->api_url = 'https://api.kashing.co.uk/';
        } else {
            $option_mode = 'test';
            $this->api_url = 'https://development-backend.kashing.co.uk/';
        }

        //Merchant ID

        $merchant_id = $this->config->get('key')[$option_mode]['merchant'];

        if (isset($merchant_id)) {
            $this->merchant_id = $merchant_id;
        } else {
            $this->add_error(array(
                'field' => 'merchant_id',
                'type' => 'missing_field',
                'msg' => t('The merchant ID is missing.')
            ));
        }

        //Secret Key

        $secret_key = $this->config->get('key')[$option_mode]['secret'];

        if (isset($secret_key)) {
            $this->secret_key = $secret_key;
        } else {
            $this->add_error(array(
                'field' => 'secret_key',
                'type' => 'missing_field',
                'msg' => t('The secret key is missing.')
            ));
        }

        //Currency
        //default set to GBP
        $currency = $this->config->get('currency') ? $this->config->get('currency') : 'GBP';

        if (isset($currency)) {
            $this->currency = $currency;
        } else {
            $this->add_error(array(
                'field' => 'currency',
                'type' => 'missing_field',
                'msg' => t('The currency ISO code is missing.')
            ));
        }


        if ($this->has_errors === false) {
            return true; // Configuration is successful
        }

        // There are errors in the plugin configuration

        return false;
    }

    private function add_error($error) {

        if ($this->has_errors == false) {
            $this->has_errors = true;
        }

        if (is_array($error)) {
            $this->errors[] = $error;
            return true;
        }

        return false;
    }

    public function hasErrors() {
        return $this->has_errors;
    }

    public function getErrors() {
        return $this->errors;
    }

    public function process($transaction_data) {

        $url = $this->api_url . 'transaction/init';

        // Return URL
        //currently returning to previous page
        $return_url = $this->get_current_url();
        $transaction_data['returnurl'] = $return_url;

        $transaction_data['merchantid'] = $this->merchant_id;
        $transaction_data['currency'] = $this->currency;

        $data_string = '';
        foreach ( $transaction_data as $data_key => $data_value ) {
            $data_string .= $data_value;
        }

        $transaction_psign = $this->get_psign($transaction_data);

        $final_transaction_array = array(
            'transactions' => array(
                array_merge(
                    $transaction_data,
                    array(
                        'psign' => $transaction_psign
                    )
                )
            )
        );


        $body = json_encode($final_transaction_array);

        $request = \Drupal::httpClient()->post($url, [
            'method' => 'POST',
            'body' => $body,
            'timeout' => 10,
            'headers' => [
                'Content-type' => 'application/json',
            ],
        ])->getBody()->getContents();

        $response_body = json_decode($request);


        if (isset($response_body->error) && isset($response_body->responsecode)) {

            if ($response_body->responsecode == 1 && isset($response_body->results)
                && isset($response_body->results[0]) && isset( $response_body->results[0]->responsecode )
                && isset( $response_body->results[0]->reasoncode ) ) {

                // redirection
                if ($response_body->results[0]->responsecode == 4 && $response_body->results[0]->reasoncode == 1
                    && isset($response_body->results) && isset($response_body->results[0]->redirect)) {

                    // redirecting the user
                    $this->redirect_url = $response_body->results[0]->redirect; // Kashing redirect URL

                    return true;

                } else { // no Redirect URL

                    $this->add_error(array(
                        'field' => 'process',
                        'type' => 'response',
                        'msg' => t('There was something wrong with a redirection response from the Kashing server.')
                    ));

                    return false;
                }
            }

            // an error

            $this->response_message = array();
            $this->response_message['response_code'] = $response_body->responsecode;
            $this->response_message['reason_code'] = $response_body->reasoncode;
            $this->response_message['error'] = $response_body->error;

            // Additional suggestion based on the error type

            $suggestion = $this->get_api_error_suggestion( $response_body->responsecode, $response_body->reasoncode );

            if ($suggestion != false) {
                $this->response_message['suggestion'] = $suggestion;
            }

            return false;
        }

        return false;
    }

    public function getRedirectUrl() {
        return $this->redirect_url;
    }

    public function getResponseMessage() {
        return $this->response_message;
    }

    private function get_api_error_suggestion($response_code, $reason_code) {

        if ($response_code == 3) {
            switch ( $reason_code ) {
                case 9:
                    return t('Please make sure your Merchant ID is correct.');
                    break;
                case 104:
                    return t('Please make sure that your Secret API Key and Merchant ID are correct.');
                    break;
            }
        }

        return '';
    }

    private function get_current_url() {

        $current_path = \Drupal::service('path.current')->getPath();
        $current_path_alias = \Drupal::service('path.alias_manager')->getAliasByPath($current_path);
        $url = Url::fromUserInput($current_path_alias, array('absolute' => TRUE))->toString();

        return $url;
    }

    private function get_psign($data_array) {

        $data_string = $this->secret_key . $this->extract_transaction_data($data_array);

        $psign = sha1($data_string);

        return $psign;
    }

    private function extract_transaction_data($data_array){

        $data_string = '';
        foreach ($data_array as $data_key => $data_value) {
            $data_string .= $data_value;
        }

        return $data_string;
    }

    public function api_get_transaction_error_details($transaction_id, $uid = null) {

        // Full API Call URL

        $url = $this->api_url . 'transaction/find';

        // Call data array

        $data_array = array(
            'token' => $transaction_id
        );

        // Psign

        $call_psign = $this->get_psign($data_array);

        // Final API Call Body with the psign (merging with the $transaction_data array)

        $final_data_array = array_merge(
            $data_array,
            array(
                'psign' => $call_psign
            )
        );

        // Encode the final transaction array to JSON

        $body = json_encode($final_data_array);

        // Make the API Call

        $request = \Drupal::httpClient()->post($url, [
            'method' => 'POST',
            'body' => $body,
            'timeout' => 10,
            'headers' => [
                'Content-type' => 'application/json',
            ],
        ])->getBody()->getContents();


        // Deal with the API response

        $response_body = json_decode($request);

        $return = array(
            'transactionID' => $transaction_id
        );

        // The gateway message

        if (isset( $response_body->gatewaymessage)) {
            if ($response_body->gatewaymessage == '') {
                $return["gatewaymessage"] = t('No additional gateway message provided.');
                $return["nogateway"] = true;
            } else {
                $return["gatewaymessage"] = Html::escape($response_body->gatewaymessage);
            }
        }

        // The reason and response codes

        if (isset($response_body->responsecode)) {
            $return["responsecode"] = Html::escape($response_body->responsecode);
        }

        if (isset( $response_body->reasoncode)) {
            $return["reasoncode"] = Html::escape($response_body->reasoncode);
        }

        // Return the array

        return $return;
    }


}